@@ -2,16 +2,14 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #define EM_VERSION_C
 
 #include "mytypes.h"
 #include "store.h"
-#include "path_max.h"
 #include "macros.h"
 #include <string.h>
 #include <stdlib.h>
@@ -24,10 +22,8 @@ extern uschar *version_date;
 void
 version_init(void)
 {
-#ifndef EXIM_BUILD_DATE_OVERRIDE
 int i = 0;
 uschar today[20];
-#endif
 
 version_string = US"2.06";
 
