@@ -3,9 +3,8 @@
 *************************************************/
 
 /* Copyright (c) University of Cambridge 1995 - 2018 */
-/* Copyright (c) The Exim Maintainers 2020 - 2021 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "../exim.h"
 
@@ -84,12 +83,13 @@ for (int i = 0; i < num_msg; i++)
     {
     case PAM_PROMPT_ECHO_ON:
     case PAM_PROMPT_ECHO_OFF:
-      if (!(arg = string_nextinlist(&pam_args, &sep, NULL, 0)))
+      arg = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
+      if (!arg)
 	{
 	arg = US"";
 	pam_arg_ended = TRUE;
 	}
-      reply[i].resp = strdup(CCS arg); /* Use libc malloc, PAM frees resp directly*/
+      reply[i].resp = CS string_copy_malloc(arg); /* PAM frees resp */
       reply[i].resp_retcode = PAM_SUCCESS;
       break;
 
@@ -155,7 +155,7 @@ pam_arg_ended = FALSE;
 fail. PAM doesn't support authentication with an empty user (it prompts for it,
 causing a potential mis-interpretation). */
 
-user = string_nextinlist(&pam_args, &sep, NULL, 0);
+user = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
 if (user == NULL || user[0] == 0) return FAIL;
 
 /* Start off PAM interaction */
