@@ -2,10 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* The idea for this code came from Matthew Byng-Maddick, but his original has
 been heavily reworked a lot for Exim 4 (and it now uses stat() (more precisely:
@@ -32,7 +31,9 @@ dsearch_open(const uschar * dirname, uschar ** errmsg)
 DIR * dp = exim_opendir(dirname);
 if (!dp)
   {
-  *errmsg = string_open_failed("%s for directory search", dirname);
+  int save_errno = errno;
+  *errmsg = string_open_failed(errno, "%s for directory search", dirname);
+  errno = save_errno;
   return NULL;
   }
 closedir(dp);
@@ -85,6 +86,10 @@ int save_errno;
 uschar * filename;
 unsigned flags = 0;
 
+handle = handle;  /* Keep picky compilers happy */
+length = length;
+do_cache = do_cache;
+
 if (Ustrchr(keystring, '/') != 0)
   {
   *errmsg = string_sprintf("key for dsearch lookup contains a slash: %s",
@@ -120,19 +125,20 @@ if (  Ulstat(filename, &statbuf) >= 0
        	 && S_ISDIR(statbuf.st_mode)
 	 && (  flags & FILTER_DIR
 	    || keystring[0] != '.'
-	    || keystring[1] && keystring[1] != '.'
+	    || keystring[1] != '.'
+	    || keystring[1] && keystring[2]
    )  )  )  )
   {
   /* Since the filename exists in the filesystem, we can return a
   non-tainted result. */
-  *result = string_copy_taint(flags & RET_FULL ? filename : keystring, GET_UNTAINTED);
+  *result = string_copy_taint(flags & RET_FULL ? filename : keystring, FALSE);
   return OK;
   }
 
-if (errno == ENOENT || errno == 0) return FAIL;
+if (errno == ENOENT) return FAIL;
 
 save_errno = errno;
-*errmsg = string_sprintf("%s: lstat: %s", filename, strerror(errno));
+*errmsg = string_sprintf("%s: lstat failed", filename);
 errno = save_errno;
 return DEFER;
 }
@@ -159,13 +165,12 @@ handle = handle;   /* Avoid compiler warning */
 
 #include "../version.h"
 
-gstring *
-dsearch_version_report(gstring * g)
+void
+dsearch_version_report(FILE *f)
 {
 #ifdef DYNLOOKUP
-g = string_fmt_append(g, "Library version: dsearch: Exim version %s\n", EXIM_VERSION_STR);
+fprintf(f, "Library version: dsearch: Exim version %s\n", EXIM_VERSION_STR);
 #endif
-return g;
 }
 
 
