@@ -2,10 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Many thanks to Stuart Lynne for contributing the original code for this
 driver. Further contributions from Michael Haardt, Brian Candler, Barry
@@ -101,10 +100,7 @@ and eldapm_find(), with a difference in the "search_type" argument.
 
 The case of eldapauth_find() is special in that all it does is do
 authentication, returning OK or FAIL as appropriate. This isn't used as a
-lookup. Instead, it is called via the generic search interface from expand.c
-as an expansion condition test.  We take a non/NULL return string as OK/FAIL.
-We do not advertise or document it as a general search method,
-but probably could.
+lookup. Instead, it is called from expand.c as an expansion condition test.
 
 The DN from a successful lookup is placed in $ldap_dn. This feature postdates
 the provision of the SEARCH_LDAP_DN facility for returning just the DN as the
@@ -117,7 +113,7 @@ Arguments:
   search_type   SEARCH_LDAP_MULTIPLE allows values from multiple entries
                 SEARCH_LDAP_SINGLE allows values from one entry only
                 SEARCH_LDAP_DN gets the DN from one entry
-  res           set to point at the result (ldapauth gets an empty string if OK)
+  res           set to point at the result (not used for ldapauth)
   errmsg        set to point a message if result is not OK
   defer_break   set TRUE if no more servers to be tried after a DEFER
   user          user name for authentication, or NULL
@@ -169,7 +165,7 @@ BOOL   attribute_found = FALSE;
 BOOL   ldapi = FALSE;
 
 DEBUG(D_lookup) debug_printf_indent("perform_ldap_search:"
-    " ldap%s URL = %q server=%s port=%d "
+    " ldap%s URL = \"%s\" server=%s port=%d "
     "sizelimit=%d timelimit=%d tcplimit=%d\n",
     search_type == SEARCH_LDAP_MULTIPLE ? "m" :
     search_type == SEARCH_LDAP_DN       ? "dn" :
@@ -181,7 +177,7 @@ library that is in use doesn't recognize, say, "ldapi", it will barf here. */
 
 if (!ldap_is_ldap_url(CS ldap_url))
   {
-  *errmsg = string_sprintf("ldap_is_ldap_url: not an LDAP url %q\n",
+  *errmsg = string_sprintf("ldap_is_ldap_url: not an LDAP url \"%s\"\n",
     ldap_url);
   goto RETURN_ERROR_BREAK;
   }
@@ -190,7 +186,7 @@ if (!ldap_is_ldap_url(CS ldap_url))
 
 if ((rc = ldap_url_parse(CS ldap_url, &ludp)) != 0)
   {
-  *errmsg = string_sprintf("ldap_url_parse: (error %d) parsing %q\n", rc,
+  *errmsg = string_sprintf("ldap_url_parse: (error %d) parsing \"%s\"\n", rc,
     ldap_url);
   goto RETURN_ERROR_BREAK;
   }
@@ -239,7 +235,7 @@ if (host)
 #if defined LDAP_LIB_OPENLDAP2
   else if (strncmp(ludp->lud_scheme, "ldapi", 5) == 0)
     {
-    *errmsg = string_sprintf("ldapi requires an absolute path (%q given)",
+    *errmsg = string_sprintf("ldapi requires an absolute path (\"%s\" given)",
       host);
     goto RETURN_ERROR;
     }
@@ -325,23 +321,21 @@ if (!lcp)
 
   else
     {
-    const uschar * init_ptr = Ustrchr(ldap_url, '/');
+    uschar * init_ptr = Ustrchr(ldap_url, '/');
     g = string_catn(NULL, ldap_url, init_ptr - ldap_url);
     g = string_fmt_append(g, "//%s:%d/", shost, port);
     }
+  string_from_gstring(g);
 
   /* Call ldap_initialize() and check the result */
-   {
-    const uschar * s = string_from_gstring(g);
 
-    DEBUG(D_lookup) debug_printf_indent("ldap_initialize with URL %s\n", s);
-    if ((rc = ldap_initialize(&ld, CS s)) != LDAP_SUCCESS)
-      {
-      *errmsg = string_sprintf("ldap_initialize: (error %d) URL %q\n",
-	rc, s);
-      goto RETURN_ERROR;
-      }
-   }
+  DEBUG(D_lookup) debug_printf_indent("ldap_initialize with URL %s\n", g->s);
+  if ((rc = ldap_initialize(&ld, CS g->s)) != LDAP_SUCCESS)
+    {
+    *errmsg = string_sprintf("ldap_initialize: (error %d) URL \"%s\"\n",
+      rc, g->s);
+    goto RETURN_ERROR;
+    }
   store_reset(reset_point);   /* Might as well save memory when we can */
 
 
@@ -502,7 +496,7 @@ if (!lcp)
 
   /* Now add this connection to the chain of cached connections */
 
-  lcp = store_get(sizeof(LDAP_CONNECTION), GET_UNTAINTED);
+  lcp = store_get(sizeof(LDAP_CONNECTION), FALSE);
   lcp->host = host ? string_copy(host) : NULL;
   lcp->bound = FALSE;
   lcp->user = NULL;
@@ -611,7 +605,6 @@ if (  !lcp->bound
 if (search_type == SEARCH_LDAP_AUTH)
   {
   DEBUG(D_lookup) debug_printf_indent("Bind succeeded: ldapauth returns OK\n");
-  *res = US"";
   goto RETURN_OK;
   }
 
@@ -1087,15 +1080,22 @@ control_ldap_search(const uschar *ldap_url, int search_type, uschar **res,
   uschar **errmsg)
 {
 BOOL defer_break = FALSE;
-int timelimit = LDAP_NO_LIMIT, sizelimit = LDAP_NO_LIMIT;
+int timelimit = LDAP_NO_LIMIT;
+int sizelimit = LDAP_NO_LIMIT;
 int tcplimit = 0;
 int sep = 0;
 int dereference = LDAP_DEREF_NEVER;
-void * referrals = LDAP_OPT_ON;
-const uschar * url = ldap_url, * p, * list;
-uschar * user = NULL, * password = NULL, * local_servers = NULL;
+void* referrals = LDAP_OPT_ON;
+const uschar *url = ldap_url;
+const uschar *p;
+uschar *user = NULL;
+uschar *password = NULL;
+uschar *local_servers = NULL;
+uschar *server;
+const uschar *list;
+uschar buffer[512];
 
-Uskip_whitespace(&url);
+while (isspace(*url)) url++;
 
 /* Until the string begins "ldap", search for the other parameter settings that
 are recognized. They are of the form NAME=VALUE, with the value being
@@ -1174,7 +1174,7 @@ while (strncmpic(url, US"ldap", 4) != 0)
         DEBUG(D_lookup) debug_printf_indent("LDAP query error: %s\n", *errmsg);
         return DEFER;
         }
-      Uskip_whitespace(&url);
+      while (isspace(*url)) url++;
       continue;
       }
     }
@@ -1251,13 +1251,13 @@ if (!eldap_default_servers && !local_servers  || p[3] != '/')
     &defer_break, user, password, sizelimit, timelimit, tcplimit, dereference,
     referrals);
 
-/* Loop through the servers until OK or FAIL. Use local_servers list
-if defined in the lookup, otherwise use the global default list */
-
-list = local_servers ? local_servers : eldap_default_servers;
-for (uschar * server; server = string_nextinlist(&list, &sep, NULL, 0); )
+/* Loop through the default servers until OK or FAIL. Use local_servers list
+ * if defined in the lookup, otherwise use the global default list */
+list = !local_servers ? eldap_default_servers : local_servers;
+while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
   {
-  int rc, port = 0;
+  int rc;
+  int port = 0;
   uschar *colon = Ustrchr(server, ':');
   if (colon)
     {
@@ -1288,6 +1288,8 @@ eldap_find(void * handle, const uschar * filename, const uschar * ldap_url,
   int length, uschar ** result, uschar ** errmsg, uint * do_cache,
   const uschar * opts)
 {
+/* Keep picky compilers happy */
+do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_SINGLE, result, errmsg));
 }
 
@@ -1296,6 +1298,8 @@ eldapm_find(void * handle, const uschar * filename, const uschar * ldap_url,
   int length, uschar ** result, uschar ** errmsg, uint * do_cache,
   const uschar * opts)
 {
+/* Keep picky compilers happy */
+do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_MULTIPLE, result, errmsg));
 }
 
@@ -1304,14 +1308,17 @@ eldapdn_find(void * handle, const uschar * filename, const uschar * ldap_url,
   int length, uschar ** result, uschar ** errmsg, uint * do_cache,
   const uschar * opts)
 {
+/* Keep picky compilers happy */
+do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_DN, result, errmsg));
 }
 
 int
 eldapauth_find(void * handle, const uschar * filename, const uschar * ldap_url,
-  int length, uschar ** result, uschar ** errmsg, uint * do_cache,
-  const uschar * opts)
+  int length, uschar ** result, uschar ** errmsg, uint * do_cache)
 {
+/* Keep picky compilers happy */
+do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_AUTH, result, errmsg));
 }
 
@@ -1341,13 +1348,16 @@ Make sure that eldap_dn does not refer to reclaimed or worse, freed store */
 static void
 eldap_tidy(void)
 {
+LDAP_CONNECTION *lcp = NULL;
 eldap_dn = NULL;
 
-for (LDAP_CONNECTION *lcp; lcp = ldap_connections; ldap_connections = lcp->next)
+while ((lcp = ldap_connections) != NULL)
   {
-  DEBUG(D_lookup) debug_printf_indent("unbind LDAP connection to %s:%d\n",
-    lcp->host, lcp->port);
-  if(lcp->bound) ldap_unbind(lcp->ld);
+  DEBUG(D_lookup) debug_printf_indent("unbind LDAP connection to %s:%d\n", lcp->host,
+    lcp->port);
+  if(lcp->bound == TRUE)
+    ldap_unbind(lcp->ld);
+  ldap_connections = lcp->next;
   }
 }
 
@@ -1408,7 +1418,6 @@ Arguments:
   s          the string to be quoted
   opt        additional option text or NULL if none
              only "dn" is recognized
-  idx	     quoter identification number
 
 Returns:     the processed string or NULL for a bad option
 */
@@ -1434,15 +1443,18 @@ quote_ldap_dn, respectively. */
 
 
 static uschar *
-eldap_quote(uschar * s, uschar * opt, unsigned idx)
+eldap_quote(uschar *s, uschar *opt)
 {
-int c, count = 0, len = 0;
+register int c;
+int count = 0;
+int len = 0;
 BOOL dn = FALSE;
-uschar * t = s, * quoted;
+uschar *t = s;
+uschar *quoted;
 
 /* Test for a DN quotation. */
 
-if (opt)
+if (opt != NULL)
   {
   if (Ustrcmp(opt, "dn") != 0) return NULL;    /* No others recognized */
   dn = TRUE;
@@ -1455,25 +1467,24 @@ where, for example, < turns into %5C%3C. For simplicity, we just add 5 for each
 possibly escaped character. The really fast way would be just to test for
 non-alphanumerics, but it is probably better to spot a few others that are
 never escaped, because if there are no specials at all, we can avoid copying
-the string.
-XXX No longer true; we always copy, to support quoted-enforcement */
+the string. */
 
-while ((c = *t++))
+while ((c = *t++) != 0)
   {
   len++;
   if (!isalnum(c) && Ustrchr(ALWAYS_LITERAL, c) == NULL) count += 5;
   }
-/*if (count == 0) return s;*/
+if (count == 0) return s;
 
 /* Get sufficient store to hold the quoted string */
 
-t = quoted = store_get_quoted(len + count + 1, s, idx, US"ldap");
+t = quoted = store_get(len + count + 1, is_tainted(s));
 
 /* Handle plain quote_ldap */
 
 if (!dn)
   {
-  while ((c = *s++))
+  while ((c = *s++) != 0)
     {
     if (!isalnum(c))
       {
@@ -1498,7 +1509,7 @@ if (!dn)
 
 else
   {
-  uschar * ss = s + len;
+  uschar *ss = s + len;
 
   /* Find the last char before any trailing spaces */
 
@@ -1522,8 +1533,8 @@ else
       {
       if (Ustrchr(LDAP_DN_QUOTE, c) != NULL)
         {
-        memcpy(t, US"%5C", 3);			/* insert \ where needed */
-        t += 3;					/* fall through to check URL */
+        Ustrncpy(t, US"%5C", 3);               /* insert \ where needed */
+        t += 3;                              /* fall through to check URL */
         }
       if (Ustrchr(URL_NONQUOTE, c) == NULL)  /* e.g. ] => %5D */
         {
@@ -1537,9 +1548,9 @@ else
 
   /* Handle the trailing spaces */
 
-  while (*ss++)
+  while (*ss++ != 0)
     {
-    memcpy(t, US"%5C%20", 6);
+    Ustrncpy(t, US"%5C%20", 6);
     t += 6;
     }
   }
@@ -1560,14 +1571,12 @@ return quoted;
 
 #include "../version.h"
 
-gstring *
-ldap_version_report(gstring * g)
+void
+ldap_version_report(FILE *f)
 {
 #ifdef DYNLOOKUP
-/*XXX it would be nice to haul a version string for the underlying ldap library */
-g = string_fmt_append(g, "Library version: LDAP: Exim version %s\n", EXIM_VERSION_STR);
+fprintf(f, "Library version: LDAP: Exim version %s\n", EXIM_VERSION_STR);
 #endif
-return g;
 }
 
 
@@ -1607,28 +1616,11 @@ static lookup_info ldapm_lookup_info = {
   .version_report = NULL                           /* no version reporting (redundant) */
 };
 
-static lookup_info ldapauth_lookup_info = {
-  .name = US"ldapauth",			/* lookup name */
-  .type = lookup_querystyle,		/* query-style lookup */
-  .open = eldap_open,			/* sic */    /* open function */
-  .check = NULL,			/* check function */
-  .find = eldapauth_find,		/* find function */
-  .close = NULL,			/* no close function */
-  .tidy = eldap_tidy,			/* sic */    /* tidy function */
-  .quote = NULL,			/* NO quoting function */
-  .version_report = NULL                           /* no version reporting (redundant) */
-};
-
 #ifdef DYNLOOKUP
 #define ldap_lookup_module_info _lookup_module_info
 #endif
 
-static lookup_info *_lookup_list[] = {
-  &ldap_lookup_info,
-  &ldapdn_lookup_info,
-  &ldapm_lookup_info,
-  &ldapauth_lookup_info,
-  };
-lookup_module_info ldap_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 4 };
+static lookup_info *_lookup_list[] = { &ldap_lookup_info, &ldapdn_lookup_info, &ldapm_lookup_info };
+lookup_module_info ldap_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 3 };
 
 /* End of lookups/ldap.c */
