@@ -2,11 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/*
- * Copyright (c) The Exim Maintainers 2015 - 2025
- * Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
  * License: GPL
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (c) The Exim Maintainers 2015 - 2020
  */
 
 /* Code for calling virus (malware) scanners. Called from acl.c. */
@@ -111,7 +109,7 @@ features_malware(void)
 {
 const uschar * s;
 uschar * t;
-uschar buf[EXIM_DRIVERNAME_MAX];
+uschar buf[64];
 
 spf(buf, sizeof(buf), US"_HAVE_MALWARE_");
 
@@ -131,7 +129,8 @@ for (const struct scan * sc = m_scans; sc->scancode != -1; sc++)
 #define MALWARE_TIMEOUT 120	/* default timeout, seconds */
 
 static const uschar * malware_regex_default = US ".+";
-static const pcre2_code * malware_default_re = NULL;
+static const pcre * malware_default_re = NULL;
+
 
 
 #ifndef DISABLE_MAL_CLAM
@@ -158,35 +157,35 @@ typedef struct clamd_address {
 # define DERR_BAD_CALL               (1<<15)  /* wrong command */
 
 static const uschar * drweb_re_str = US "infected\\swith\\s*(.+?)$";
-static const pcre2_code * drweb_re = NULL;
+static const pcre * drweb_re = NULL;
 #endif
 
 #ifndef DISABLE_MAL_FSECURE
 static const uschar * fsec_re_str = US "\\S{0,5}INFECTED\\t[^\\t]*\\t([^\\t]+)\\t\\S*$";
-static const pcre2_code * fsec_re = NULL;
+static const pcre * fsec_re = NULL;
 #endif
 
 #ifndef DISABLE_MAL_KAV
 static const uschar * kav_re_sus_str = US "suspicion:\\s*(.+?)\\s*$";
 static const uschar * kav_re_inf_str = US "infected:\\s*(.+?)\\s*$";
-static const pcre2_code * kav_re_sus = NULL;
-static const pcre2_code * kav_re_inf = NULL;
+static const pcre * kav_re_sus = NULL;
+static const pcre * kav_re_inf = NULL;
 #endif
 
 #ifndef DISABLE_MAL_AVAST
 static const uschar * ava_re_clean_str = US "(?!\\\\)\\t\\[\\+\\]";
 static const uschar * ava_re_virus_str = US "(?!\\\\)\\t\\[L\\]\\d+\\.0\\t0\\s(.*)";
 static const uschar * ava_re_error_str = US "(?!\\\\)\\t\\[E\\]\\d+\\.0\\tError\\s\\d+\\s(.*)";
-static const pcre2_code * ava_re_clean = NULL;
-static const pcre2_code * ava_re_virus = NULL;
-static const pcre2_code * ava_re_error = NULL;
+static const pcre * ava_re_clean = NULL;
+static const pcre * ava_re_virus = NULL;
+static const pcre * ava_re_error = NULL;
 #endif
 
 #ifndef DISABLE_MAL_FFROT6D
 static const uschar * fprot6d_re_error_str = US "^\\d+\\s<(.+?)>$";
 static const uschar * fprot6d_re_virus_str = US "^\\d+\\s<infected:\\s+(.+?)>\\s+.+$";
-static const pcre2_code * fprot6d_re_error = NULL;
-static const pcre2_code * fprot6d_re_virus = NULL;
+static const pcre * fprot6d_re_error = NULL;
+static const pcre * fprot6d_re_virus = NULL;
 #endif
 
 
@@ -221,7 +220,6 @@ extern uschar spooled_message_id[MESSAGE_ID_LENGTH+1];
 /* Some (currently avast only) use backslash escaped whitespace,
 this function undoes these escapes */
 
-#ifndef DISABLE_MAL_AVAST
 static inline void
 unescape(uschar *p)
 {
@@ -230,7 +228,6 @@ for (; *p; ++p)
   if (*p == '\\' && (isspace(p[1]) || p[1] == '\\'))
     for (p0 = p; *p0; ++p0) *p0 = p0[1];
 }
-#endif
 
 /* --- malware_*_defer --- */
 static inline int
@@ -253,6 +250,13 @@ m_panic_defer(struct scan * scanent, const uschar * hostport,
 return malware_panic_defer(string_sprintf("%s %s : %s",
   scanent->name, hostport ? hostport : CUS"", str));
 }
+static inline int
+m_log_defer(struct scan * scanent, const uschar * hostport,
+  const uschar * str)
+{
+return malware_log_defer(string_sprintf("%s %s : %s",
+  scanent->name, hostport ? hostport : CUS"", str));
+}
 /* --- m_*_defer_3 */
 static inline int
 m_panic_defer_3(struct scan * scanent, const uschar * hostport,
@@ -273,15 +277,8 @@ static inline int
 m_tcpsocket(const uschar * hostname, unsigned int port,
 	host_item * host, uschar ** errstr, const blob * fastopen_blob)
 {
-int fd = ip_connectedsocket(SOCK_STREAM, hostname, port, port, 5,
+return ip_connectedsocket(SOCK_STREAM, hostname, port, port, 5,
 			  host, errstr, fastopen_blob);
-#ifdef EXIM_TFO_FREEBSD
-/* Under some fault conditions, FreeBSD 12.2 seen to send a (non-TFO) SYN
-and, getting no response, wait for a long time.  Impose a 5s max. */
-if (fd >= 0)
-  (void) poll_one_fd(fd, POLLOUT, 5 * 1000);
-#endif
-return fd;
 }
 #endif
 
@@ -299,36 +296,37 @@ if (send(sock, buf, cnt, 0) < 0)
 return sock;
 }
 
-static const pcre2_code *
-m_pcre_compile(const uschar * re, BOOL cacheable, uschar ** errstr)
+static const pcre *
+m_pcre_compile(const uschar * re, uschar ** errstr)
 {
-return regex_compile(re, cacheable ? MCS_CACHEABLE : MCS_NOFLAGS, errstr,
-		      pcre_gen_cmp_ctx);
+const uschar * rerror;
+int roffset;
+const pcre * cre;
+
+if (!(cre = pcre_compile(CS re, PCRE_COPT, CCSS &rerror, &roffset, NULL)))
+  *errstr= string_sprintf("regular expression error in '%s': %s at offset %d",
+      re, rerror, roffset);
+return cre;
 }
 
 uschar *
-m_pcre_exec(const pcre2_code * cre, uschar * text)
+m_pcre_exec(const pcre * cre, uschar * text)
 {
-pcre2_match_data * md = pcre2_match_data_create(2, pcre_gen_ctx);
-int i = pcre2_match(cre, text, PCRE2_ZERO_TERMINATED, 0, 0, md, pcre_gen_mtc_ctx);
+int ovector[10*3];
+int i = pcre_exec(cre, NULL, CS text, Ustrlen(text), 0, 0,
+	      ovector, nelem(ovector));
 uschar * substr = NULL;
-
 if (i >= 2)				/* Got it */
-  {
-  PCRE2_SIZE * ovec = pcre2_get_ovector_pointer(md);
-  int len = ovec[3] - ovec[2];
-  substr = string_copyn(text + ovec[2], len);
-  }
-/* pcre2_match_data_free(md);	gen ctx needs no free */
+  pcre_get_substring(CS text, ovector, i, 1, CCSS &substr);
 return substr;
 }
 
-static const pcre2_code *
+static const pcre *
 m_pcre_nextinlist(const uschar ** list, int * sep,
- BOOL cacheable, char * listerr, uschar ** errstr)
+ char * listerr, uschar ** errstr)
 {
 const uschar * list_ele;
-const pcre2_code * cre = NULL;
+const pcre * cre = NULL;
 
 if (!(list_ele = string_nextinlist(list, sep, NULL, 0)))
   *errstr = US listerr;
@@ -336,7 +334,7 @@ else
   {
   DEBUG(D_acl) debug_printf_indent("%15s%10s'%s'\n", "", "RE: ",
     string_printing(list_ele));
-  cre = m_pcre_compile(CUS list_ele, cacheable, errstr);
+  cre = m_pcre_compile(CUS list_ele, errstr);
   }
 return cre;
 }
@@ -388,7 +386,6 @@ return p - buffer;
 }
 
 /* return TRUE iff size as requested */
-#ifndef DISABLE_MAL_DRWEB
 static BOOL
 recv_len(int sock, void * buf, int size, time_t tmo)
 {
@@ -396,7 +393,6 @@ return fd_ready(sock, tmo)
   ? recv(sock, buf, size, 0) == size
   : FALSE;
 }
-#endif
 
 
 
@@ -562,7 +558,6 @@ is via malware(), or there's malware_in_file() used for testing/debugging.
 
 Arguments:
   malware_re    match condition for "malware="
-  cacheable	the RE did not use any dynamic elements during expansion
   scan_filename  the file holding the email to be scanned, if we're faking
 		this up for the -bmalware test, else NULL
   timeout	if nonzero, non-default timeoutl
@@ -571,22 +566,21 @@ Returns:        Exim message processing code (OK, FAIL, DEFER, ...)
                 where true means malware was found (condition applies)
 */
 static int
-malware_internal(const uschar * malware_re, BOOL cacheable,
-  const uschar * scan_filename, int timeout)
+malware_internal(const uschar * malware_re, const uschar * scan_filename,
+  int timeout)
 {
 int sep = 0;
 const uschar *av_scanner_work = av_scanner;
-BOOL av_scanner_textonly;
 uschar *scanner_name;
 unsigned long mbox_size;
 FILE *mbox_file;
-const pcre2_code *re;
+const pcre *re;
 uschar * errstr;
 struct scan * scanent;
 const uschar * scanner_options;
 client_conn_ctx malware_daemon_ctx = {.sock = -1};
 time_t tmo;
-const uschar * eml_filename, * eml_dir;
+uschar * eml_filename, * eml_dir;
 
 if (!malware_re)
   return FAIL;		/* empty means "don't match anything" */
@@ -603,30 +597,30 @@ the name), so we can close it right away.  Get the directory too. */
 eml_dir = string_copyn(eml_filename, Ustrrchr(eml_filename, '/') - eml_filename);
 
 /* parse 1st option */
-if (strcmpic(malware_re, US"false") == 0  ||  Ustrcmp(malware_re, "0") == 0)
+if (strcmpic(malware_re, US"false") == 0  ||  Ustrcmp(malware_re,"0") == 0)
   return FAIL;		/* explicitly no matching */
 
 /* special cases (match anything except empty) */
-if (  strcmpic(malware_re, US"true") == 0
-   || Ustrcmp(malware_re, "*") == 0
-   || Ustrcmp(malware_re, "1") == 0
+if (  strcmpic(malware_re,US"true") == 0
+   || Ustrcmp(malware_re,"*") == 0
+   || Ustrcmp(malware_re,"1") == 0
    )
   {
   if (  !malware_default_re
-     && !(malware_default_re = m_pcre_compile(malware_regex_default, FALSE, &errstr)))
+     && !(malware_default_re = m_pcre_compile(malware_regex_default, &errstr)))
     return malware_panic_defer(errstr);
   malware_re = malware_regex_default;
   re = malware_default_re;
   }
 
 /* compile the regex, see if it works */
-else if (!(re = m_pcre_compile(malware_re, cacheable, &errstr)))
+else if (!(re = m_pcre_compile(malware_re, &errstr)))
   return malware_panic_defer(errstr);
 
 /* if av_scanner starts with a dollar, expand it first */
 if (*av_scanner == '$')
   {
-  if (!(av_scanner_work = expand_string_2(av_scanner, &av_scanner_textonly)))
+  if (!(av_scanner_work = expand_string(av_scanner)))
     return malware_panic_defer(
 	 string_sprintf("av_scanner starts with $, but expansion failed: %s",
 	 expand_string_message));
@@ -637,8 +631,6 @@ if (*av_scanner == '$')
   malware_name = NULL;
   malware_ok = FALSE;
   }
-else
-  av_scanner_textonly = TRUE;
 
 /* Do not scan twice (unless av_scanner is dynamic). */
 if (!malware_ok)
@@ -743,11 +735,13 @@ if (!malware_ok)
     case M_FPROT6D: /* "f-prot6d" scanner type ----------------------------------- */
       {
       int bread;
-      uschar * e, * linebuffer, * scanrequest;
+      uschar * e;
+      uschar * linebuffer;
+      uschar * scanrequest;
       uschar av_buffer[1024];
 
-      if ((!fprot6d_re_virus && !(fprot6d_re_virus = m_pcre_compile(fprot6d_re_virus_str, FALSE, &errstr)))
-        || (!fprot6d_re_error && !(fprot6d_re_error = m_pcre_compile(fprot6d_re_error_str, FALSE, &errstr))))
+      if ((!fprot6d_re_virus && !(fprot6d_re_virus = m_pcre_compile(fprot6d_re_virus_str, &errstr)))
+        || (!fprot6d_re_error && !(fprot6d_re_error = m_pcre_compile(fprot6d_re_error_str, &errstr))))
         return malware_panic_defer(errstr);
 
       scanrequest = string_sprintf("SCAN FILE %s\n", eml_filename);
@@ -917,12 +911,12 @@ badseek:  err = errno;
 
 	/* set up match regex */
 	if (!drweb_re)
-	  drweb_re = m_pcre_compile(drweb_re_str, FALSE, &errstr);
+	  drweb_re = m_pcre_compile(drweb_re_str, &errstr);
 
 	/* read and concatenate virus names into one string */
 	for (int i = 0; i < drweb_vnum; i++)
 	  {
-	  pcre2_match_data * md = pcre2_match_data_create(2, pcre_gen_ctx);
+	  int ovector[10*3];
 
 	  /* read the size of report */
 	  if (!recv_len(malware_daemon_ctx.sock, &drweb_slen, sizeof(drweb_slen), tmo))
@@ -931,7 +925,7 @@ badseek:  err = errno;
 	  drweb_slen = ntohl(drweb_slen);
 
 	  /* assume tainted, since it is external input */
-	  tmpbuf = store_get(drweb_slen, GET_TAINTED);
+	  tmpbuf = store_get(drweb_slen, TRUE);
 
 	  /* read report body */
 	  if (!recv_len(malware_daemon_ctx.sock, tmpbuf, drweb_slen, tmo))
@@ -940,22 +934,23 @@ badseek:  err = errno;
 	  tmpbuf[drweb_slen] = '\0';
 
 	  /* try matcher on the line, grab substring */
-	  result = pcre2_match(drweb_re, (PCRE2_SPTR)tmpbuf, PCRE2_ZERO_TERMINATED,
-				0, 0, md, pcre_gen_mtc_ctx);
+	  result = pcre_exec(drweb_re, NULL, CS tmpbuf, Ustrlen(tmpbuf), 0, 0,
+				  ovector, nelem(ovector));
 	  if (result >= 2)
 	    {
-	    PCRE2_SIZE * ovec = pcre2_get_ovector_pointer(md);
+	    const char * pre_malware_nb;
+
+	    pcre_get_substring(CS tmpbuf, ovector, result, 1, &pre_malware_nb);
 
 	    if (i==0)	/* the first name we just copy to malware_name */
-	      g = string_catn(NULL, US ovec[2], ovec[3] - ovec[2]);
+	      g = string_cat(NULL, US pre_malware_nb);
 
+	    /*XXX could be string_append_listele? */
 	    else	/* concatenate each new virus name to previous */
-	      {
-	      g = string_catn(g, US"/", 1);
-	      g = string_catn(g, US ovec[2], ovec[3] - ovec[2]);
-	      }
+	      g = string_append(g, 2, "/", pre_malware_nb);
+
+	    pcre_free_substring(pre_malware_nb);
 	    }
-	  /* pcre2_match_data_free(md);	gen ctx needs no free */
 	  }
 	  malware_name = string_from_gstring(g);
 	}
@@ -1096,7 +1091,7 @@ badseek:  err = errno;
       /* set up match */
       /* todo also SUSPICION\t */
       if (!fsec_re)
-	fsec_re = m_pcre_compile(fsec_re_str, FALSE, &errstr);
+	fsec_re = m_pcre_compile(fsec_re_str, &errstr);
 
       /* read report, linewise. Apply a timeout as the Fsecure daemon
       sometimes wants an answer to "PING" but they won't tell us what */
@@ -1147,7 +1142,7 @@ badseek:  err = errno;
       int kav_rc;
       unsigned long kav_reportlen;
       int bread;
-      const pcre2_code *kav_re;
+      const pcre *kav_re;
       uschar *p;
 
       /* get current date and time, build scan request */
@@ -1220,12 +1215,12 @@ badseek:  err = errno;
 	    /* set up match regex, depends on retcode */
 	    if (kav_rc == 3)
 	      {
-	      if (!kav_re_sus) kav_re_sus = m_pcre_compile(kav_re_sus_str, FALSE, &errstr);
+	      if (!kav_re_sus) kav_re_sus = m_pcre_compile(kav_re_sus_str, &errstr);
 	      kav_re = kav_re_sus;
 	      }
 	    else
 	      {
-	      if (!kav_re_inf) kav_re_inf = m_pcre_compile(kav_re_inf_str, FALSE, &errstr);
+	      if (!kav_re_inf) kav_re_inf = m_pcre_compile(kav_re_inf_str, &errstr);
 	      kav_re = kav_re_inf;
 	      }
 
@@ -1256,8 +1251,8 @@ badseek:  err = errno;
     case M_CMDL: /* "cmdline" scanner type ---------------------------------- */
       {
       const uschar *cmdline_scanner = scanner_options;
-      const pcre2_code *cmdline_trigger_re;
-      const pcre2_code *cmdline_regex_re;
+      const pcre *cmdline_trigger_re;
+      const pcre *cmdline_regex_re;
       uschar * file_name;
       uschar * commandline;
       void (*eximsigchld)(int);
@@ -1274,13 +1269,13 @@ badseek:  err = errno;
 	return m_panic_defer(scanent, NULL, errstr);
 
       /* find scanner output trigger */
-      cmdline_trigger_re = m_pcre_nextinlist(&av_scanner_work, &sep, av_scanner_textonly,
+      cmdline_trigger_re = m_pcre_nextinlist(&av_scanner_work, &sep,
 				"missing trigger specification", &errstr);
       if (!cmdline_trigger_re)
 	return m_panic_defer(scanent, NULL, errstr);
 
       /* find scanner name regex */
-      cmdline_regex_re = m_pcre_nextinlist(&av_scanner_work, &sep, av_scanner_textonly,
+      cmdline_regex_re = m_pcre_nextinlist(&av_scanner_work, &sep,
 			  "missing virus name regex specification", &errstr);
       if (!cmdline_regex_re)
 	return m_panic_defer(scanent, NULL, errstr);
@@ -1451,7 +1446,9 @@ badseek:  err = errno;
       uschar av_buffer[1024];
       uschar *hostname = US"";
       host_item connhost;
-      int clam_fd;
+      uschar *clamav_fbuf;
+      int clam_fd, result;
+      off_t fsize;
       unsigned int fsize_uint;
       BOOL use_scan_command = FALSE;
       clamd_address * cv[MAX_CLAMD_SERVERS];
@@ -1469,9 +1466,9 @@ badseek:  err = errno;
 	int subsep = ' ';
 
 	/* Local file; so we def want to use_scan_command and don't want to try
-	passing IP/port combinations */
+	 * passing IP/port combinations */
 	use_scan_command = TRUE;
-	cd = (clamd_address *) store_get(sizeof(clamd_address), GET_UNTAINTED);
+	cd = (clamd_address *) store_get(sizeof(clamd_address), FALSE);
 
 	/* extract socket-path part */
 	sublist = scanner_options;
@@ -1505,7 +1502,7 @@ badseek:  err = errno;
 	    continue;
 	    }
 
-	  cd = (clamd_address *) store_get(sizeof(clamd_address), GET_UNTAINTED);
+	  cd = (clamd_address *) store_get(sizeof(clamd_address), FALSE);
 
 	  /* extract host and port part */
 	  sublist = scanner_options;
@@ -1558,17 +1555,16 @@ badseek:  err = errno;
 	{ cmd_str.data = US"zINSTREAM"; cmd_str.len = 10; }
       else
 	{
-	int n;
-	cmd_str.data = string_sprintf("SCAN %s\n%n", eml_filename, &n);
-	cmd_str.len = n;		/* .len is a size_t */
+	cmd_str.data = string_sprintf("SCAN %s\n", eml_filename);
+	cmd_str.len = Ustrlen(cmd_str.data);
 	}
 
       /* We have some network servers specified */
       if (num_servers)
 	{
 	/* Confirmed in ClamAV source (0.95.3) that the TCPAddr option of clamd
-	only supports AF_INET, but we should probably be looking to the
-	future and rewriting this to be protocol-independent anyway. */
+	 * only supports AF_INET, but we should probably be looking to the
+	 * future and rewriting this to be protocol-independent anyway. */
 
 	while (num_servers > 0)
 	  {
@@ -1579,17 +1575,16 @@ badseek:  err = errno;
 			 cd->hostspec, cd->tcp_port);
 
 	  /* Lookup the host. This is to ensure that we connect to the same IP
-	  on both connections (as one host could resolve to multiple ips) */
+	   * on both connections (as one host could resolve to multiple ips) */
 	  for (;;)
 	    {
-	    /*XXX we trust that the cmd_str is idempotent */
+	    /*XXX we trust that the cmd_str is ideempotent */
 	    if ((malware_daemon_ctx.sock = m_tcpsocket(cd->hostspec, cd->tcp_port,
-				    &connhost, &errstr,
-				    use_scan_command ? &cmd_str : NULL)) >= 0)
+				    &connhost, &errstr, &cmd_str)) >= 0)
 	      {
 	      /* Connection successfully established with a server */
 	      hostname = cd->hostspec;
-	      if (use_scan_command) cmd_str.len = 0;
+	      cmd_str.len = 0;
 	      break;
 	      }
 	    if (cd->retry <= 0) break;
@@ -1623,28 +1618,20 @@ badseek:  err = errno;
 	  }
 
       /* have socket in variable "sock"; command to use is semi-independent of
-      the socket protocol.  We use SCAN if is local (either Unix/local
-      domain socket, or explicitly told local) else we stream the data.
-      How we stream the data depends upon how we were built.  */
+       * the socket protocol.  We use SCAN if is local (either Unix/local
+       * domain socket, or explicitly told local) else we stream the data.
+       * How we stream the data depends upon how we were built.  */
 
       if (!use_scan_command)
 	{
-	struct stat st;
-#if defined(EXIM_TCP_CORK) && !defined(OS_SENDFILE)
-	BOOL corked = TRUE;
-#endif
 	/* New protocol: "zINSTREAM\n" followed by a sequence of <length><data>
 	chunks, <n> a 4-byte number (network order), terminated by a zero-length
-	chunk. We only send one chunk. */
+	chunk. */
 
 	DEBUG(D_acl) debug_printf_indent(
 	    "Malware scan: issuing %s new-style remote scan (zINSTREAM)\n",
 	    scanner_name);
 
-#if defined(EXIM_TCP_CORK)
-	(void) setsockopt(malware_daemon_ctx.sock, IPPROTO_TCP, EXIM_TCP_CORK,
-			  US &on, sizeof(on));
-#endif
 	/* Pass the string to ClamAV (10 = "zINSTREAM\0"), if not already sent */
 	if (cmd_str.len)
 	  if (send(malware_daemon_ctx.sock, cmd_str.data, cmd_str.len, 0) < 0)
@@ -1653,6 +1640,7 @@ badseek:  err = errno;
 		strerror(errno)),
 	      malware_daemon_ctx.sock);
 
+	/* calc file size */
 	if ((clam_fd = exim_open2(CS eml_filename, O_RDONLY)) < 0)
 	  {
 	  int err = errno;
@@ -1661,79 +1649,66 @@ badseek:  err = errno;
 	      eml_filename, strerror(err)),
 	    malware_daemon_ctx.sock);
 	  }
-	if (fstat(clam_fd, &st) < 0)
+	if ((fsize = lseek(clam_fd, 0, SEEK_END)) < 0)
 	  {
-	  int err = errno;
+	  int err;
+b_seek:   err = errno;
 	  (void)close(clam_fd);
 	  return m_panic_defer_3(scanent, NULL,
-	    string_sprintf("can't stat spool file %s: %s",
+	    string_sprintf("can't seek spool file %s: %s",
 	      eml_filename, strerror(err)),
 	    malware_daemon_ctx.sock);
 	  }
-	fsize_uint = (unsigned int) st.st_size;
-	if ((off_t)fsize_uint != st.st_size)
+	fsize_uint = (unsigned int) fsize;
+	if ((off_t)fsize_uint != fsize)
 	  {
 	  (void)close(clam_fd);
 	  return m_panic_defer_3(scanent, NULL,
-	    string_sprintf("stat spool file %s, size overflow", eml_filename),
+	    string_sprintf("seeking spool file %s, size overflow",
+	      eml_filename),
 	    malware_daemon_ctx.sock);
 	  }
+	if (lseek(clam_fd, 0, SEEK_SET) < 0)
+	  goto b_seek;
 
-	/* send file size */
-	send_size = htonl(fsize_uint);
-	if (send(malware_daemon_ctx.sock, &send_size, sizeof(send_size), 0) < 0)
-	  return m_panic_defer_3(scanent, NULL,
-	    string_sprintf("unable to send file size to socket (%s)", hostname),
-	    malware_daemon_ctx.sock);
-
-	/* send file body */
-	while (fsize_uint)
+	if (!(clamav_fbuf = store_malloc(fsize_uint)))
 	  {
-#ifdef OS_SENDFILE
-	  int n = os_sendfile(malware_daemon_ctx.sock, clam_fd, NULL, (size_t)fsize_uint);
-	  if (n < 0)
-	    return m_panic_defer_3(scanent, NULL,
-	      string_sprintf("unable to send file body to socket (%s): %s", hostname, strerror(errno)),
-	      malware_daemon_ctx.sock);
-	  fsize_uint -= n;
-#else
-	  int n = MIN(fsize_uint, big_buffer_size);
-	  if ((n = read(clam_fd, big_buffer, n)) < 0)
-	    return m_panic_defer_3(scanent, NULL,
-	      string_sprintf("can't read spool file %s: %s",
-		eml_filename, strerror(errno)),
-	      malware_daemon_ctx.sock);
-	  if (send(malware_daemon_ctx.sock, big_buffer, (size_t)n, 0) < 0)
-	    return m_panic_defer_3(scanent, NULL,
-	      string_sprintf("unable to send file body to socket (%s): %s", hostname, strerror(errno)),
-	      malware_daemon_ctx.sock);
-	  fsize_uint -= n;
-# ifdef EXIM_TCP_CORK
-	  if (corked)
-	    {
-	    corked = FALSE;
-	    (void) setsockopt(malware_daemon_ctx.sock, IPPROTO_TCP, EXIM_TCP_CORK,
-			      US &off, sizeof(off));
-	    }
-# endif
-#endif	/*!OS_SENDFILE*/
-
+	  (void)close(clam_fd);
+	  return m_panic_defer_3(scanent, NULL,
+	    string_sprintf("unable to allocate memory %u for file (%s)",
+	      fsize_uint, eml_filename),
+	    malware_daemon_ctx.sock);
 	  }
 
-	send_final_zeroblock = 0;
-	if (send(malware_daemon_ctx.sock, &send_final_zeroblock, sizeof(send_final_zeroblock), 0) < 0)
+	if ((result = read(clam_fd, clamav_fbuf, fsize_uint)) < 0)
+	  {
+	  int err = errno;
+	  store_free(clamav_fbuf); (void)close(clam_fd);
 	  return m_panic_defer_3(scanent, NULL,
-	    string_sprintf("unable to send file terminator to socket (%s)", hostname),
+	    string_sprintf("can't read spool file %s: %s",
+	      eml_filename, strerror(err)),
 	    malware_daemon_ctx.sock);
-#ifdef OS_SENDFILE
-	(void) setsockopt(malware_daemon_ctx.sock, IPPROTO_TCP, EXIM_TCP_CORK,
-			  US &off, sizeof(off));
-#endif
+	  }
+	(void)close(clam_fd);
+
+	/* send file body to socket */
+	send_size = htonl(fsize_uint);
+	send_final_zeroblock = 0;
+	if ((send(malware_daemon_ctx.sock, &send_size, sizeof(send_size), 0) < 0) ||
+	    (send(malware_daemon_ctx.sock, clamav_fbuf, fsize_uint, 0) < 0) ||
+	    (send(malware_daemon_ctx.sock, &send_final_zeroblock, sizeof(send_final_zeroblock), 0) < 0))
+	  {
+	  store_free(clamav_fbuf);
+	  return m_panic_defer_3(scanent, NULL,
+	    string_sprintf("unable to send file body to socket (%s)", hostname),
+	    malware_daemon_ctx.sock);
+	  }
+	store_free(clamav_fbuf);
 	}
       else
 	{ /* use scan command */
 	/* Send a SCAN command pointing to a filename; then in the then in the
-	scan-method-neutral part, read the response back */
+	 * scan-method-neutral part, read the response back */
 
 /* ================================================================= */
 
@@ -1758,10 +1733,10 @@ badseek:  err = errno;
 	      malware_daemon_ctx.sock);
 
 	/* Do not shut down the socket for writing; a user report noted that
-	clamd 0.70 does not react well to this. */
+	 * clamd 0.70 does not react well to this. */
 	}
       /* Commands have been sent, no matter which scan method or connection
-      type we're using; now just read the result, independent of method. */
+       * type we're using; now just read the result, independent of method. */
 
       /* Read the result */
       memset(av_buffer, 0, sizeof(av_buffer));
@@ -1807,7 +1782,6 @@ badseek:  err = errno;
       /* strip newline at the end (won't be present for zINSTREAM)
       (also any trailing whitespace, which shouldn't exist, but we depend upon
       this below, so double-check) */
-
       p = av_buffer + Ustrlen(av_buffer) - 1;
       if (*p == '\n') *p = '\0';
 
@@ -1818,7 +1792,7 @@ badseek:  err = errno;
       if (*p) ++p;
 
       /* colon in returned output? */
-      if (!(p = Ustrchr(av_buffer,':')))
+      if(!(p = Ustrchr(av_buffer,':')))
 	return m_panic_defer(scanent, CUS callout_address, string_sprintf(
 		  "ClamAV returned malformed result (missing colon): %s",
 		  av_buffer));
@@ -1848,7 +1822,7 @@ badseek:  err = errno;
 	    *p = '\0';
 	  }
 	malware_name = string_copy(vname);
-	DEBUG(D_acl) debug_printf_indent("Malware found, name %q\n", malware_name);
+	DEBUG(D_acl) debug_printf_indent("Malware found, name \"%s\"\n", malware_name);
 
 	}
       else if (Ustrcmp(result_tag, "ERROR") == 0)
@@ -1882,8 +1856,8 @@ badseek:  err = errno;
       uschar * linebuffer;
       uschar * sockline_scanner;
       uschar sockline_scanner_default[] = "%s\n";
-      const pcre2_code *sockline_trig_re;
-      const pcre2_code *sockline_name_re;
+      const pcre *sockline_trig_re;
+      const pcre *sockline_name_re;
 
       /* find scanner command line */
       if (  (sockline_scanner = string_nextinlist(&av_scanner_work, &sep,
@@ -1903,19 +1877,19 @@ badseek:  err = errno;
 	string_printing(sockline_scanner));
 
       /* find scanner output trigger */
-      sockline_trig_re = m_pcre_nextinlist(&av_scanner_work, &sep, av_scanner_textonly,
+      sockline_trig_re = m_pcre_nextinlist(&av_scanner_work, &sep,
 				"missing trigger specification", &errstr);
       if (!sockline_trig_re)
 	return m_panic_defer_3(scanent, NULL, errstr, malware_daemon_ctx.sock);
 
       /* find virus name regex */
-      sockline_name_re = m_pcre_nextinlist(&av_scanner_work, &sep, av_scanner_textonly,
+      sockline_name_re = m_pcre_nextinlist(&av_scanner_work, &sep,
 			  "missing virus name regex specification", &errstr);
       if (!sockline_name_re)
 	return m_panic_defer_3(scanent, NULL, errstr, malware_daemon_ctx.sock);
 
       /* prepare scanner call - security depends on expansions check above */
-      commandline = string_sprintf( CS sockline_scanner, eml_filename);
+      commandline = string_sprintf( CS sockline_scanner, CS eml_filename);
       DEBUG(D_acl) debug_printf_indent("%15s%10s'%s'\n", "", "expanded: ",
 	string_printing(commandline));
 
@@ -2040,11 +2014,11 @@ badseek:  err = errno;
        */
 
       if (  (  !ava_re_clean
-            && !(ava_re_clean = m_pcre_compile(ava_re_clean_str, FALSE, &errstr)))
+            && !(ava_re_clean = m_pcre_compile(ava_re_clean_str, &errstr)))
 	 || (  !ava_re_virus
-	    && !(ava_re_virus = m_pcre_compile(ava_re_virus_str, FALSE, &errstr)))
+	    && !(ava_re_virus = m_pcre_compile(ava_re_virus_str, &errstr)))
 	 || (  !ava_re_error
-	    && !(ava_re_error = m_pcre_compile(ava_re_error_str, FALSE, &errstr)))
+	    && !(ava_re_error = m_pcre_compile(ava_re_error_str, &errstr)))
 	 )
 	return malware_panic_defer(errstr);
 
@@ -2122,7 +2096,7 @@ badseek:  err = errno;
               if (malware_name)     /* Nothing else matters, just read on */
                 break;
 
-	      if (regex_match(ava_re_clean, buf, slen, NULL))
+	      if (pcre_exec(ava_re_clean, NULL, CS buf, slen, 0, 0, NULL, 0) == 0)
 		break;
 
               if ((malware_name = m_pcre_exec(ava_re_virus, buf)))
@@ -2143,7 +2117,7 @@ badseek:  err = errno;
                   break;
                   }
                 }
-              else if (regex_match(ava_re_error, buf, slen, NULL))
+              else if (pcre_exec(ava_re_error, NULL, CS buf, slen, 0, 0, NULL, 0) == 0)
                 {
                 log_write(0, LOG_MAIN, "internal scanner error (ignored): %s", buf);
                 break;
@@ -2178,7 +2152,6 @@ badseek:  err = errno;
 
       }
 #endif
-  default:	break;	/* compiler quietening */
   }	/* scanner type switch */
 
   if (malware_daemon_ctx.sock >= 0)
@@ -2207,16 +2180,15 @@ filename; it's a wrapper around the malware_file function.
 
 Arguments:
   malware_re  match condition for "malware="
-  cacheable   the RE did not use any dynamic elements during expansion
   timeout     if nonzero, timeout in seconds
 
 Returns:      Exim message processing code (OK, FAIL, DEFER, ...)
               where true means malware was found (condition applies)
 */
 int
-malware(const uschar * malware_re, BOOL cacheable, int timeout)
+malware(const uschar * malware_re, int timeout)
 {
-int ret = malware_internal(malware_re, cacheable, NULL, timeout);
+int ret = malware_internal(malware_re, NULL, timeout);
 
 if (ret == DEFER) av_failed = TRUE;
 return ret;
@@ -2239,7 +2211,7 @@ Returns:        Exim message processing code (OK, FAIL, DEFER, ...)
                 where true means malware was found (condition applies)
 */
 int
-malware_in_file(const uschar * eml_filename)
+malware_in_file(uschar *eml_filename)
 {
 uschar message_id_buf[64];
 int ret;
@@ -2256,9 +2228,9 @@ recipients_list = NULL;
 receive_add_recipient(US"malware-victim@example.net", -1);
 f.enable_dollar_recipients = TRUE;
 
-ret = malware_internal(US"*", TRUE, eml_filename, 0);
+ret = malware_internal(US"*", eml_filename, 0);
 
-memcpy(spooled_message_id, message_id, sizeof(spooled_message_id));
+Ustrncpy(spooled_message_id, message_id, sizeof(spooled_message_id));
 spool_mbox_ok = 1;
 
 /* don't set no_mbox_unspool; at present, there's no way for it to become
@@ -2277,35 +2249,35 @@ void
 malware_init(void)
 {
 if (!malware_default_re)
-  malware_default_re = regex_must_compile(malware_regex_default, MCS_NOFLAGS, TRUE);
+  malware_default_re = regex_must_compile(malware_regex_default, FALSE, TRUE);
 
 #ifndef DISABLE_MAL_DRWEB
 if (!drweb_re)
-  drweb_re = regex_must_compile(drweb_re_str, MCS_NOFLAGS, TRUE);
+  drweb_re = regex_must_compile(drweb_re_str, FALSE, TRUE);
 #endif
 #ifndef DISABLE_MAL_FSECURE
 if (!fsec_re)
-  fsec_re = regex_must_compile(fsec_re_str, MCS_NOFLAGS, TRUE);
+  fsec_re = regex_must_compile(fsec_re_str, FALSE, TRUE);
 #endif
 #ifndef DISABLE_MAL_KAV
 if (!kav_re_sus)
-  kav_re_sus = regex_must_compile(kav_re_sus_str, MCS_NOFLAGS, TRUE);
+  kav_re_sus = regex_must_compile(kav_re_sus_str, FALSE, TRUE);
 if (!kav_re_inf)
-  kav_re_inf = regex_must_compile(kav_re_inf_str, MCS_NOFLAGS, TRUE);
+  kav_re_inf = regex_must_compile(kav_re_inf_str, FALSE, TRUE);
 #endif
 #ifndef DISABLE_MAL_AVAST
 if (!ava_re_clean)
-  ava_re_clean = regex_must_compile(ava_re_clean_str, MCS_NOFLAGS, TRUE);
+  ava_re_clean = regex_must_compile(ava_re_clean_str, FALSE, TRUE);
 if (!ava_re_virus)
-  ava_re_virus = regex_must_compile(ava_re_virus_str, MCS_NOFLAGS, TRUE);
+  ava_re_virus = regex_must_compile(ava_re_virus_str, FALSE, TRUE);
 if (!ava_re_error)
-  ava_re_error = regex_must_compile(ava_re_error_str, MCS_NOFLAGS, TRUE);
+  ava_re_error = regex_must_compile(ava_re_error_str, FALSE, TRUE);
 #endif
 #ifndef DISABLE_MAL_FFROT6D
 if (!fprot6d_re_error)
-  fprot6d_re_error = regex_must_compile(fprot6d_re_error_str, MCS_NOFLAGS, TRUE);
+  fprot6d_re_error = regex_must_compile(fprot6d_re_error_str, FALSE, TRUE);
 if (!fprot6d_re_virus)
-  fprot6d_re_virus = regex_must_compile(fprot6d_re_virus_str, MCS_NOFLAGS, TRUE);
+  fprot6d_re_virus = regex_must_compile(fprot6d_re_virus_str, FALSE, TRUE);
 #endif
 }
 
