@@ -2,10 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Functions for writing spool files, and moving them about. */
 
@@ -52,7 +51,7 @@ if (f) (void)fclose(f);
 if (errmsg)
   *errmsg = msg;
 else
-  log_write_die(0, LOG_MAIN, "%s", msg);
+  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s", msg);
 
 return -1;
 }
@@ -106,31 +105,11 @@ return fd;
 
 
 
-static const uschar *
-zap_newlines(const uschar *s)
-{
-uschar *z, *p;
-
-if (Ustrchr(s, '\n') == NULL) return s;
-
-p = z = string_copy(s);
-while ((p = Ustrchr(p, '\n')) != NULL) *p++ = ' ';
-return z;
-}
-
 static void
 spool_var_write(FILE * fp, const uschar * name, const uschar * val)
 {
-putc('-', fp);
-if (is_tainted(val))
-  {
-  const uschar * quoter_name;
-  putc('-', fp);
-  (void) quoter_for_address(val, &quoter_name);
-  if (quoter_name)
-    fprintf(fp, "(%s)", quoter_name);
-  }
-fprintf(fp, "%s %s\n", name, val);
+if (is_tainted(val)) putc('-', fp);
+fprintf(fp, "-%s %s\n", name, val);
 }
 
 /*************************************************
@@ -145,7 +124,7 @@ be open and locked, thus preventing any other exim process from working on this
 message.
 
 Argument:
-  id      the message id (used for the eventual filename; the *content* uses the global. Unclear why.)
+  id      the message id
   where   SW_RECEIVING, SW_DELIVERING, or SW_MODIFYING
   errmsg  where to put an error message; if NULL, panic-die on error
 
@@ -154,13 +133,16 @@ Returns:  the size of the header texts on success;
 */
 
 int
-spool_write_header(const uschar * id, int where, uschar ** errmsg)
+spool_write_header(uschar *id, int where, uschar **errmsg)
 {
-int fd, size_correction;
+int fd;
+int size_correction;
 FILE * fp;
 struct stat statbuf;
+uschar * tname;
 uschar * fname;
-uschar * tname = spool_fname(US"input", message_subdir, US"hdr.", message_id);
+
+tname = spool_fname(US"input", message_subdir, US"hdr.", message_id);
 
 if ((fd = spool_open_temp(tname)) < 0)
   return spool_write_error(where, errmsg, US"open", NULL, NULL);
@@ -180,8 +162,6 @@ fprintf(fp, "<%s>\n", sender_address);
 fprintf(fp, "%d %d\n", (int)received_time.tv_sec, warning_count);
 
 fprintf(fp, "-received_time_usec .%06d\n", (int)received_time.tv_usec);
-fprintf(fp, "-received_time_complete %d.%06d\n",
-  (int)received_time_complete.tv_sec, (int)received_time_complete.tv_usec);
 
 /* If there is information about a sending host, remember it. The HELO
 data can be set for local SMTP as well as remote. */
@@ -191,21 +171,19 @@ if (sender_helo_name) spool_var_write(fp, US"helo_name", sender_helo_name);
 if (sender_host_address)
   {
   if (is_tainted(sender_host_address)) putc('-', fp);
-  fprintf(fp, "-host_address [%s]:%d\n", sender_host_address, sender_host_port);
+  fprintf(fp, "-host_address %s.%d\n", sender_host_address, sender_host_port);
   if (sender_host_name)
     spool_var_write(fp, US"host_name", sender_host_name);
+  if (sender_host_authenticated)
+    spool_var_write(fp, US"host_auth", sender_host_authenticated);
   }
-if (sender_host_authenticated)
-  spool_var_write(fp, US"host_auth", sender_host_authenticated);
-if (sender_host_auth_pubname)
-  spool_var_write(fp, US"host_auth_pubname", sender_host_auth_pubname);
 
 /* Also about the interface a message came in on */
 
 if (interface_address)
   {
   if (is_tainted(interface_address)) putc('-', fp);
-  fprintf(fp, "-interface_address [%s]:%d\n", interface_address, interface_port);
+  fprintf(fp, "-interface_address %s.%d\n", interface_address, interface_port);
   }
 
 if (smtp_active_hostname != primary_hostname)
@@ -230,12 +208,6 @@ tree_walk(acl_var_m, &acl_var_write, fp);
 
 /* Now any other data that needs to be remembered. */
 
-if (*debuglog_name)
-  {
-  fprintf(fp, "-debug_selector 0x%x\n", debug_selector);
-  fprintf(fp, "-debuglog_name %s\n", debuglog_name);
-  }
-
 if (f.spool_file_wireformat)
   fprintf(fp, "-spool_file_wireformat\n");
 else
@@ -247,7 +219,7 @@ if (body_zerocount > 0) fprintf(fp, "-body_zerocount %d\n", body_zerocount);
 if (authenticated_id)
   spool_var_write(fp, US"auth_id", authenticated_id);
 if (authenticated_sender)
-  spool_var_write(fp, US"auth_sender", zap_newlines(authenticated_sender));
+  spool_var_write(fp, US"auth_sender", authenticated_sender);
 
 if (f.allow_unqualified_recipient) fprintf(fp, "-allow_unqualified_recipient\n");
 if (f.allow_unqualified_sender) fprintf(fp, "-allow_unqualified_sender\n");
@@ -289,7 +261,7 @@ if (tls_in.ourcert)
     fprintf(fp, "-tls_ourcert %s\n", CS big_buffer);
   }
 if (tls_in.ocsp)	 fprintf(fp, "-tls_ocsp %d\n",   tls_in.ocsp);
-# ifndef DISABLE_TLS_RESUME
+# ifdef EXPERIMENTAL_TLS_RESUME
 fprintf(fp, "-tls_resumption %c\n", 'A' + tls_in.resumption);
 # endif
 if (tls_in.ver) spool_var_write(fp, US"tls_ver", tls_in.ver);
@@ -320,20 +292,19 @@ fprintf(fp, "%d\n", recipients_count);
 for (int i = 0; i < recipients_count; i++)
   {
   recipient_item *r = recipients_list + i;
-  const uschar *address = zap_newlines(r->address);
 
   /* DEBUG(D_deliver) debug_printf("DSN: Flags: 0x%x\n", r->dsn_flags); */
 
   if (r->pno < 0 && !r->errors_to && r->dsn_flags == 0)
-    fprintf(fp, "%s\n", address);
+    fprintf(fp, "%s\n", r->address);
   else
     {
-    const uschar *errors_to = r->errors_to ? zap_newlines(r->errors_to) : CUS"";
+    uschar * errors_to = r->errors_to ? r->errors_to : US"";
     /* for DSN SUPPORT extend exim 4 spool in a compatible way by
     adding new values upfront and add flag 0x02 */
-    const uschar *orcpt = r->orcpt ? zap_newlines(r->orcpt) : CUS"";
+    uschar * orcpt = r->orcpt ? r->orcpt : US"";
 
-    fprintf(fp, "%s %s %d,%d %s %d,%d#3\n", address, orcpt, Ustrlen(orcpt),
+    fprintf(fp, "%s %s %d,%d %s %d,%d#3\n", r->address, orcpt, Ustrlen(orcpt),
       r->dsn_flags, errors_to, Ustrlen(errors_to), r->pno);
     }
 
@@ -446,7 +417,7 @@ start-up time.
 
 Arguments:
   dir        base directory name
-  dq	     destination queue name
+  dq	     destiinationqueue name
   subdir     subdirectory name
   id         message id
   suffix     suffix to add to id
@@ -459,14 +430,14 @@ Returns:     TRUE if all went well
 */
 
 static BOOL
-make_link(const uschar * dir, const uschar * dq, const uschar * subdir, const uschar * id,
-  const uschar * suffix, const uschar * from, const uschar * to, BOOL noentok)
+make_link(uschar *dir, uschar * dq, uschar *subdir, uschar *id, uschar *suffix,
+  uschar *from, uschar *to, BOOL noentok)
 {
 uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suffix);
 uschar * tname = spool_q_fname(string_sprintf("%s%s", to,   dir), dq, subdir, id, suffix);
 if (Ulink(fname, tname) < 0 && (!noentok || errno != ENOENT))
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "link(%q, %q) failed while moving "
+  log_write(0, LOG_MAIN|LOG_PANIC, "link(\"%s\", \"%s\") failed while moving "
     "message: %s", fname, tname, strerror(errno));
   return FALSE;
   }
@@ -496,13 +467,13 @@ Returns:     TRUE if all went well
 */
 
 static BOOL
-break_link(const uschar * dir, const uschar * subdir, const uschar * id,
-  const uschar * suffix, const uschar * from, BOOL noentok)
+break_link(uschar *dir, uschar *subdir, uschar *id, uschar *suffix, uschar *from,
+  BOOL noentok)
 {
 uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suffix);
 if (Uunlink(fname) < 0 && (!noentok || errno != ENOENT))
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "unlink(%q) failed while moving "
+  log_write(0, LOG_MAIN|LOG_PANIC, "unlink(\"%s\") failed while moving "
     "message: %s", fname, strerror(errno));
   return FALSE;
   }
@@ -530,14 +501,10 @@ Returns:      TRUE if all is well
 */
 
 BOOL
-spool_move_message(const uschar * id, const uschar * subdir,
-  const uschar * from, const uschar * to)
+spool_move_message(uschar *id, uschar *subdir, uschar *from, uschar *to)
 {
 uschar * dest_qname = queue_name_dest ? queue_name_dest : queue_name;
 
-/* Since we are working within the spool, de-taint the dest queue name */
-dest_qname = string_copy_taint(dest_qname, GET_UNTAINTED);
-
 /* Create any output directories that do not exist. */
 
 (void) directory_make(spool_directory,
