@@ -2,13 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
-#ifndef STRUCTS_H
-#define STRUCTS_H
 
 /* Definitions of various structures. In addition, those that are visible for
 the compilation of local_scan() are defined in local_scan.h. These are
@@ -64,7 +61,7 @@ typedef struct ugid_block {
   BOOL    initgroups;
 } ugid_block;
 
-typedef enum {	CHUNKING_NOT_OFFERED = 0,
+typedef enum {	CHUNKING_NOT_OFFERED = -1,
 		CHUNKING_OFFERED,
 		CHUNKING_ACTIVE,
 		CHUNKING_LAST} chunking_state_t;
@@ -79,30 +76,19 @@ typedef enum {	TFO_NOT_USED = 0,
 but also used when checking lists of hosts and when transporting. Looking up
 host addresses is done using this structure. */
 
-typedef enum {DS_UNK, DS_NO, DS_YES} dnssec_status_t;
-
-#ifdef EXPERIMENTAL_SRV_SMTPS
-typedef enum {SRV_TLS_UNK, SRV_STARTTLS_CAN, SRV_STARTTLS_MUST,
-	      SRV_TLS_ON_CONNECT} srv_tls_t;
-#endif
+typedef enum {DS_UNK=-1, DS_NO, DS_YES} dnssec_status_t;
 
 typedef struct host_item {
   struct host_item *next;
-  const uschar *name;		/* Host name */
-#ifndef DISABLE_TLS
-  const uschar *certname;	/* Name used for certificate checks */
-#endif
-  const uschar *address;	/* IP address in text form */
-  int		port;		/* port value in host order (if SRV lookup) */
-  int		mx;		/* MX value if found via MX records */
-  uschar	status;		/* Usable, unusable, or unknown */
-  uschar	why;		/* Why host is unusable */
-  dnssec_status_t dnssec_used:2;
-#ifdef EXPERIMENTAL_SRV_SMTPS
-  srv_tls_t	  tls_needs:2;
-#endif
-  int		sort_key;		/* MX*1000 plus random "fraction" */
-  time_t	last_try;		/* Time of last try if known */
+  const uschar *name;             /* Host name */
+  const uschar *address;          /* IP address in text form */
+  int     port;                   /* port value in host order (if SRV lookup) */
+  int     mx;                     /* MX value if found via MX records */
+  int     sort_key;               /* MX*1000 plus random "fraction" */
+  int     status;                 /* Usable, unusable, or unknown */
+  int     why;                    /* Why host is unusable */
+  int     last_try;               /* Time of last try if known */
+  dnssec_status_t dnssec;
 } host_item;
 
 /* Chain of rewrite rules, read from the rewrite config, or parsed from the
@@ -140,39 +126,23 @@ three, the layout of the start of the blocks is kept the same, and represented
 by the generic structures driver_info and driver_instance. */
 
 typedef struct driver_instance {
-  void   *next;
+  struct driver_instance *next;
   uschar *name;                   /* Instance name */
-  void   *info;			  /* Points to info for this driver */
+  struct driver_info *info;       /* Points to info for this driver */
   void   *options_block;          /* Pointer to private options */
-
   uschar *driver_name;            /* All start with this generic option */
-  const uschar *srcfile;	  /* and config source info for errors */
-  int	  srcline;
 } driver_instance;
 
 typedef struct driver_info {
-  struct driver_info * next;
   uschar *driver_name;            /* Name of driver */
-
   optionlist *options;            /* Table of private options names */
   int    *options_count;          /* -> Number of entries in table */
   void   *options_block;          /* Points to default private block */
   int     options_len;            /* Length of same in bytes */
   void  (*init)(                  /* Initialization entry point */
-	  struct driver_instance *);
-  uint	  dyn_magic;		  /* Magic num if dynamic, else zero */
+    struct driver_instance *);
 } driver_info;
 
-/* Values for dyn_magic.  Encode types and api version. */
-#define ROUTER_MAGIC	0x52544d31	/* RTM1 */
-#define TRANSPORT_MAGIC	0x54504d31	/* TPM1 */
-#define AUTH_MAGIC	0x65554d31	/* AUM1 */
-
-typedef struct {
-  uint		magic;
-  uschar *	class;
-} driver_magics;
-
 
 /* Structure for holding information about the configured transports. Some
 of the generally accessible options are set from the configuration file; others
@@ -181,8 +151,11 @@ transports. They need to be generally accessible, however, as they are used by
 the main transport code. */
 
 typedef struct transport_instance {
-  driver_instance drinst;
-
+  struct transport_instance *next;
+  uschar *name;                   /* Instance name */
+  struct transport_info *info;    /* Info for this driver */
+  void *options_block;            /* Pointer to private options */
+  uschar *driver_name;            /* Must be first */
   int   (*setup)(                 /* Setup entry point */
     struct transport_instance *,
     struct address_item *,
@@ -199,7 +172,7 @@ typedef struct transport_instance {
   uschar *expand_multi_domain;    /* )                                  */
   BOOL    multi_domain;           /* )                                  */
   BOOL    overrides_hosts;        /* ) Used only for remote transports  */
-  uschar *max_addresses;          /* )                                  */
+  int     max_addresses;          /* )                                  */
   int     connection_max_messages;/* )                                  */
                                   /**************************************/
   BOOL    deliver_as_creator;     /* Used only by pipe at present */
@@ -243,12 +216,18 @@ typedef struct transport_instance {
 } transport_instance;
 
 
-/* Structure for holding information about a type of transport.  The first
-element must be a struct driver_info, to match auths and routers. */
+/* Structure for holding information about a type of transport. The first six
+fields must match driver_info above. */
 
 typedef struct transport_info {
-  driver_info drinfo;
-
+  uschar *driver_name;            /* Driver name */
+  optionlist *options;            /* Table of private options names */
+  int    *options_count;          /* -> Number of entries in table */
+  void   *options_block;          /* Points to default private block */
+  int     options_len;            /* Length of same in bytes */
+  void (*init)(                   /* Initialization function */
+    struct transport_instance *);
+/****/
   BOOL (*code)(                   /* Main entry point */
     transport_instance *,
     struct address_item *);
@@ -296,7 +275,11 @@ typedef struct {
 /* Structure for holding information about the configured routers. */
 
 typedef struct router_instance {
-  driver_instance drinst;
+  struct router_instance *next;
+  uschar *name;
+  struct router_info *info;
+  void   *options_block;          /* Pointer to private options */
+  uschar *driver_name;            /* Must be first */
 
   uschar *address_data;           /* Arbitrary data */
 #ifdef EXPERIMENTAL_BRIGHTMAIL
@@ -373,12 +356,18 @@ typedef struct router_instance {
 } router_instance;
 
 
-/* Structure for holding information about a type of router.  The first element
-must be a struct driver_info, to match auths and transports. */
+/* Structure for holding information about a type of router. The first six
+fields must match driver_info above. */
 
 typedef struct router_info {
-  driver_info drinfo;
-
+  uschar *driver_name;
+  optionlist *options;            /* Table of private options names */
+  int    *options_count;          /* -> Number of entries in table */
+  void   *options_block;          /* Points to default private block */
+  int     options_len;            /* Length of same in bytes */
+  void (*init)(                   /* Initialization function */
+    struct router_instance *);
+/****/
   int (*code)(                    /* Main entry point */
     router_instance *,
     struct address_item *,
@@ -403,8 +392,11 @@ typedef struct router_info {
 mechanisms */
 
 typedef struct auth_instance {
-  driver_instance drinst;
-
+  struct auth_instance *next;
+  uschar *name;                   /* Exim instance name */
+  struct auth_info *info;         /* Pointer to driver info block */
+  void   *options_block;          /* Pointer to private options */
+  uschar *driver_name;            /* Must be first */
   uschar *advertise_condition;    /* Are we going to advertise this?*/
   uschar *client_condition;       /* Should the client try this? */
   uschar *public_name;            /* Advertised name */
@@ -420,11 +412,17 @@ typedef struct auth_instance {
 
 
 /* Structure for holding information about an authentication mechanism. The
-first element must be a struct driver_info, to match routers and transports. */
+first six fields must match driver_info above. */
 
 typedef struct auth_info {
-  driver_info drinfo;
-
+  uschar *driver_name;            /* e.g. "condition" */
+  optionlist *options;            /* Table of private options names */
+  int    *options_count;          /* -> Number of entries in table */
+  void   *options_block;          /* Points to default private block */
+  int     options_len;            /* Length of same in bytes */
+  void (*init)(                   /* initialization function */
+    struct auth_instance *);
+/****/
   int (*servercode)(              /* server function */
     auth_instance *,              /* the instance data */
     uschar *);                    /* rest of AUTH command */
@@ -434,8 +432,8 @@ typedef struct auth_info {
     int,                          /* command timeout */
     uschar *,                     /* buffer for reading response */
     int);                         /* sizeof buffer */
-  gstring * (*version_report)(    /* diagnostic version reporting */
-    gstring *);                   /* string to append to */
+  void (*version_report)(         /* diagnostic version reporting */
+    FILE *);                      /* I/O stream to print to */
   void (*macros_create)(void);	  /* feature-macro creation */
 } auth_info;
 
@@ -455,8 +453,8 @@ typedef struct ip_address_item {
 /* Structure for chaining together arbitrary strings. */
 
 typedef struct string_item {
-  struct string_item *	next;
-  uschar *		text;
+  struct string_item *next;
+  uschar *text;
 } string_item;
 
 /* Information about a soft delivery failure, for use when calculating
@@ -466,7 +464,7 @@ can be tried. */
 
 typedef struct retry_item {
   struct retry_item *next;        /* for chaining */
-  const uschar *key;              /* string identifying host/address/message */
+  uschar *key;                    /* string identifying host/address/message */
   int     basic_errno;            /* error code for this destination */
   int     more_errno;             /* additional error information */
   uschar *message;                /* local error message */
@@ -512,11 +510,14 @@ typedef struct address_item_propagated {
   uschar *address_data;           /* arbitrary data to keep with the address */
   uschar *domain_data;            /* from "domains" lookup */
   uschar *localpart_data;         /* from "local_parts" lookup */
-  const uschar *errors_address;         /* where to send errors (NULL => sender) */
+  uschar *errors_address;         /* where to send errors (NULL => sender) */
   header_line *extra_headers;     /* additional headers */
   uschar *remove_headers;         /* list of those to remove */
   void   *variables;		  /* router-vasriables */
 
+#ifdef EXPERIMENTAL_SRS
+  uschar *srs_sender;             /* Change return path when delivering */
+#endif
   BOOL    ignore_error:1;	  /* ignore delivery error */
 #ifdef SUPPORT_I18N
   BOOL    utf8_msg:1;		  /* requires SMTPUTF8 processing */
@@ -547,18 +548,17 @@ typedef struct address_item {
   reply_item *reply;              /* data for autoreply */
   retry_item *retries;            /* chain of retry information */
 
-  const uschar *address;                /* address being delivered or routed */
+  uschar *address;                /* address being delivered or routed */
   uschar *unique;                 /* used for disambiguating */
-  const uschar *cc_local_part;    /* caseful local part */
-  const uschar *lc_local_part;    /* lowercased local part */
-  const uschar *local_part;       /* points to cc or lc version */
-  const uschar *prefix;           /* stripped prefix of local part */
-  const uschar *prefix_v;	  /*  variable part of above */
-  const uschar *suffix;           /* stripped suffix of local part */
-  const uschar *suffix_v;	  /*  variable part of above */
+  uschar *cc_local_part;          /* caseful local part */
+  uschar *lc_local_part;          /* lowercased local part */
+  uschar *local_part;             /* points to cc or lc version */
+  uschar *prefix;                 /* stripped prefix of local part */
+  uschar *prefix_v;		  /*  variable part of above */
+  uschar *suffix;                 /* stripped suffix of local part */
+  uschar *suffix_v;		  /*  variable part of above */
   const uschar *domain;           /* working domain (lower cased) */
 
-  const uschar *return_path;	  /* mailfrom, for logging */
   uschar *address_retry_key;      /* retry key including full address */
   uschar *domain_retry_key;       /* retry key for domain only */
 
@@ -567,17 +567,12 @@ typedef struct address_item {
   uschar *message;                /* error message */
   uschar *user_message;           /* error message that can be sent over SMTP
                                      or quoted in bounce message */
-  const uschar *onetime_parent;         /* saved original parent for onetime */
+  uschar *onetime_parent;         /* saved original parent for onetime */
   uschar **pipe_expandn;          /* numeric expansions for pipe from filter */
   uschar *return_filename;        /* name of return file */
   uschar *self_hostname;          /* after self=pass */
   uschar *shadow_message;         /* info about shadow transporting */
 
-  uid_t   uid;                    /* uid for transporting */
-  gid_t   gid;                    /* gid for transporting */
-
-  gstring *protocol_sequence;	  /* smtp session start sequence */
-
 #ifndef DISABLE_TLS
   const uschar *tlsver;           /* version used for transport */
   uschar *cipher;                 /* Cipher used for transport */
@@ -600,9 +595,9 @@ typedef struct address_item {
   int     dsn_flags;              /* DSN flags */
   int     dsn_aware;              /* DSN aware flag */
 
-#ifndef DISABLE_DKIM
-  const uschar * dkim_used;	  /* DKIM info, or NULL */
-#endif
+  uid_t   uid;                    /* uid for transporting */
+  gid_t   gid;                    /* gid for transporting */
+
   				  /* flags */
   struct {
     BOOL af_allow_file:1;		/* allow file in generated address */
@@ -628,8 +623,6 @@ typedef struct address_item {
     BOOL af_verify_routed:1;		/* for cached sender verify: routed OK */
     BOOL af_verify_callout:1;		/* for cached sender verify: callout was specified */
     BOOL af_include_affixes:1;		/* delivered with affixes in RCPT */
-    BOOL af_new_conn:1;			/* delivered on an fresh TCP conn */
-    BOOL af_cont_conn:1;		/* delivered (with new MAIL cmd) on an existing TCP conn */
     BOOL af_cert_verified:1;		/* delivered with verified TLS cert */
     BOOL af_pass_message:1;		/* pass message in bounces */
     BOOL af_bad_reply:1;		/* filter could not generate autoreply */
@@ -651,7 +644,7 @@ typedef struct address_item {
 #ifdef SUPPORT_I18N
     BOOL af_utf8_downcvt:1;		/* downconvert was done for delivery */
 #endif
-#ifndef DISABLE_TLS_RESUME
+#ifdef EXPERIMENTAL_TLS_RESUME
     BOOL af_tls_resume:1;		/* TLS used a resumed session */
 #endif
   } flags;
@@ -710,14 +703,12 @@ typedef struct retry_rule {
 } retry_rule;
 
 typedef struct retry_config {
-  struct retry_config *	next;
-  int			srcline;
-  const uschar *	srcfile;
-  uschar *		pattern;
-  int			basic_errno;
-  int			more_errno;
-  uschar *		senders;
-  retry_rule *		rules;
+  struct retry_config *next;
+  uschar *pattern;
+  int     basic_errno;
+  int     more_errno;
+  uschar *senders;
+  retry_rule *rules;
 } retry_config;
 
 /* Structure for each node in a tree, of which there are various kinds */
@@ -757,7 +748,7 @@ to close. */
 
 typedef struct search_cache {
   void   *handle;                 /* lookup handle, or NULL if closed */
-  const lookup_info * li;	  /* info struct for search type */
+  int search_type;                /* search type */
   tree_node *up;                  /* LRU up pointer */
   tree_node *down;                /* LRU down pointer */
   tree_node *item_cache;          /* tree of cached results */
@@ -820,19 +811,14 @@ typedef struct sha1 {
 
 /* Information for making an smtp connection */
 typedef struct {
-  transport_instance *	tblock;
+  transport_instance *  tblock;
   void *		ob;	/* smtp_transport_options_block * */
-  host_item *		host;
-  int			host_af;
-  const uschar *	interface;
-
-  int			sock;	/* used for a bound but not connected socket */
-  uschar *		sending_ip_address;	/* used for TLS resumption */
-  const uschar *	host_lbserver;		/* ditto, for server-behind LB */
-  BOOL			have_lbserver:1;	/* host_lbserver is valid */
+  host_item *           host;
+  int                   host_af;
+  uschar *              interface;
 
 #ifdef SUPPORT_DANE
-  BOOL			dane:1;		/* TLSA says connection must do dane */
+  BOOL dane:1;			/* connection must do dane */
   dns_answer		tlsa_dnsa;	/* strictly, this should use tainted mem */
 #endif
 } smtp_connect_args;
@@ -884,23 +870,13 @@ typedef struct redirect_block {
   BOOL    check_group;            /* TRUE, FALSE, or TRUE_UNSET */
 } redirect_block;
 
-/* Sieve control data */
-
-typedef struct sieve_block {
-  const uschar * inbox;
-  const uschar * enotify_mailto_owner;
-  const uschar * subaddress;
-  const uschar * useraddress;
-  const uschar * vacation_dir;
-} sieve_block;
-
 /* Structure for passing arguments to check_host() */
 
 typedef struct check_host_block {
   const uschar *host_name;
   const uschar *host_address;
   const uschar *host_ipv4;
-  mcs_flags	flags;
+  BOOL   negative;
 } check_host_block;
 
 /* Structure for remembering lookup data when caching the result of
@@ -962,96 +938,4 @@ struct ob_dkim {
 #endif
 };
 
-
-/* per-queue-runner info */
-typedef struct qrunner {
-  struct qrunner * next;	/* list sorted by next tick */
-
-  uschar *	name;		/* NULL for the default queue */
-  unsigned	interval;	/* tick rate, seconds. Zero for a one-time run */
-  time_t	next_tick;	/* next run should, or should have, start(ed) */
-  unsigned	run_max;	/* concurrent queue runner limit */
-  unsigned	run_count;	/* current runners */
-
-  BOOL queue_run_force :1;
-  BOOL deliver_force_thaw :1;
-  BOOL queue_run_first_delivery :1;
-  BOOL queue_run_local :1;
-  BOOL queue_2stage :1;
-} qrunner;
-
-
-/* Types of variable table entry */
-
-enum vtypes {
-  vtype_int,            /* value is address of int */
-  vtype_filter_int,     /* ditto, but recognized only when filtering */
-  vtype_ino,            /* value is address of ino_t (not always an int) */
-  vtype_uid,            /* value is address of uid_t (not always an int) */
-  vtype_gid,            /* value is address of gid_t (not always an int) */
-  vtype_bool,           /* value is address of bool */
-  vtype_stringptr,      /* value is address of pointer to string */
-  vtype_msgbody,        /* as stringptr, but read when first required */
-  vtype_msgbody_end,    /* ditto, the end of the message */
-  vtype_msgheaders,     /* the message's headers, processed */
-  vtype_msgheaders_raw, /* the message's headers, unprocessed */
-  vtype_localpart,      /* extract local part from string */
-  vtype_domain,         /* extract domain from string */
-  vtype_string_func,	/* value is string returned by given function */
-  vtype_todbsdin,       /* value not used; generate BSD inbox tod */
-  vtype_tode,           /* value not used; generate tod in epoch format */
-  vtype_todel,          /* value not used; generate tod in epoch/usec format */
-  vtype_todf,           /* value not used; generate full tod */
-  vtype_todl,           /* value not used; generate log tod */
-  vtype_todlf,          /* value not used; generate log file datestamp tod */
-  vtype_todzone,        /* value not used; generate time zone only */
-  vtype_todzulu,        /* value not used; generate zulu tod */
-  vtype_reply,          /* value not used; get reply from headers */
-  vtype_pid,            /* value not used; result is pid */
-  vtype_host_lookup,    /* value not used; get host name */
-  vtype_load_avg,       /* value not used; result is int from os_getloadavg */
-  vtype_pspace,         /* partition space; value is T/F for spool/log */
-  vtype_pinodes,        /* partition inodes; value is T/F for spool/log */
-  vtype_cert,		/* SSL certificate */
-#ifndef DISABLE_DKIM
-  vtype_dkim,           /* Lookup of value in DKIM signature */
-#endif
-  vtype_module,		/* variable lives in a module; value is module name */
-};
-
-/* Type for main variable table */
-
-typedef struct {
-  const char *name;
-  enum vtypes type;
-  void       *value;
-} var_entry;
-
-
-
-/* dynamic-load module info */
-
-typedef struct misc_module_info {
-  struct misc_module_info * next;
-
-  const uschar * name;
-  unsigned	dyn_magic;
-  BOOL		(*init)(void *);	/* arg is the misc_module_info ptr */
-  gstring *	(*lib_vers_report)(gstring *);	/* underlying library */
-  int		(*conn_init)(const uschar *, const uschar *, const uschar * *);
-  void		(*smtp_reset)(void);
-  int		(*msg_init)(void);
-  gstring *	(*authres)(gstring *);
-
-  void *	options;
-  unsigned	options_count;
-  void *	functions;
-  unsigned	functions_count;
-  void *	variables;
-  unsigned	variables_count;
-} misc_module_info;
-
-#define MISC_MODULE_MAGIC	0x4d4d4d31	/* MMM1 */
-
-#endif	/* whole file */
 /* End of structs.h */
